/*
 *  Copyright (c) by Jaroslav Kysela <perex@jcu.cz>
 */

#include "driver.h"
#include "gus.h"

static void ultra_gus_free_card( void *ptr )
{
  ultra_gus_card_t *gus = (ultra_gus_card_t *)ptr;
  ultra_free( gus, sizeof( ultra_gus_card_t ) );
}

ultra_gus_card_t *ultra_gus_new_card( ultra_card_t *card )
{
  ultra_gus_card_t *gus;
  
  gus = (ultra_gus_card_t *)ultra_malloc( sizeof( ultra_gus_card_t ) );
  if ( !gus ) return NULL;
  MEMSET( gus, 0, sizeof( ultra_gus_card_t ) );
  gus -> card = card;
  card -> private = gus;
  card -> private_free = ultra_gus_free_card;
  return gus;
}

int ultra_gus_set_port( ultra_gus_card_t *gus, unsigned short port )
{
  gus -> gf1.port = port;
  /* fill register variables for speedup */
  gus -> gf1.reg_page = GUSP( gus, GF1PAGE );
  gus -> gf1.reg_regsel = GUSP( gus, GF1REGSEL );
  gus -> gf1.reg_data8 = GUSP( gus, GF1DATAHIGH );
  gus -> gf1.reg_data16 = GUSP( gus, GF1DATALOW );
  gus -> gf1.reg_irqstat = GUSP( gus, IRQSTAT );
  gus -> gf1.reg_dram = GUSP( gus, DRAM );
  gus -> gf1.reg_timerctrl = GUSP( gus, TIMERCNTRL );
  gus -> gf1.reg_timerdata = GUSP( gus, TIMERDATA );
  /* ---- */
  return 0;
}

/*
 *  Memory detection routine for plain GF1 soundcards
 */

int ultra_gus_detect_memory( ultra_gus_card_t *gus )
{
  int l, idx, local;
  unsigned char d;

  ultra_gf1_poke( gus, 0L, 0xaa );
  ultra_gf1_poke( gus, 1L, 0x55 );
  if ( ultra_gf1_peek( gus, 0L ) != 0xaa || ultra_gf1_peek( gus, 1L ) != 0x55 ) {
    PRINTK( "ultra: plain GF1 card at 0x%x without onboard DRAM?\n", gus -> gf1.port );
    return -ENOMEM;
  }
  for ( idx = 1, d = 0xaa; idx < 4; idx++, d++ ) {
    local = idx << 18;
    ultra_gf1_poke( gus, local, d );
    ultra_gf1_poke( gus, local + 1, d + 1 );
    if ( ultra_gf1_peek( gus, local ) != d ||
         ultra_gf1_peek( gus, local + 1 ) != d + 1 ||
         ultra_gf1_peek( gus, 0L ) != 0xaa ) break;
  }
#if 1
  gus -> gf1.memory = idx << 18;
#else
  gus -> gf1.memory = 256 * 1024;
#endif
  for ( l = 0, local = gus -> gf1.memory; l < 4; l++, local -= 256 * 1024 ) {
    gus -> gf1.banks_8[ l ].address =
    gus -> gf1.banks_8[ l ].size = 0;
    gus -> gf1.banks_16[ l ].address = l << 18;
    gus -> gf1.banks_16[ l ].size = local > 0 ? 256 * 1024 : 0;
  }
  gus -> gf1.banks_8[ 0 ].size = gus -> gf1.memory;
  return 0;		/* some memory were detected */
}

int ultra_gus_init_dma_irq( ultra_gus_card_t *gus, int latches )
{
  ultra_card_t *card;
  unsigned long flags;
  unsigned char i, irq, dma;
  static unsigned char irqs[ 16 ] = { 0, 0, 1, 3, 0, 2, 0, 4, 0, 1, 0, 5, 6, 0, 0, 7 };
  static unsigned char dmas[ 8 ] = { 6, 1, 0, 2, 0, 3, 4, 5 };

  if ( !gus ) return -EINVAL;
  card = gus -> card;
  if ( !card ) return -EINVAL;

  gus -> mix_cntrl_reg &= 0xf8;
  gus -> mix_cntrl_reg |= 0x01;	/* disable MIC, LINE IN, enable LINE OUT */
  if ( gus -> ics_flag || gus -> codec_flag || gus -> ess_flag ) {
    gus -> mix_cntrl_reg &= ~1;	/* enable LINE IN */
    gus -> mix_cntrl_reg |= 4;	/* enable MIC */
  }

  dma = dmas[ card -> dmas[ ULTRA_GUS_DMA_GPLAY ].dma & 7 ];
  i = dmas[ card -> dmas[ ULTRA_GUS_DMA_GRECORD ].dma & 7 ];
#if 0
  printk( "dma1 = %i, dma2 = %i\n", card -> dmas[ ULTRA_GUS_DMA_GPLAY ].dma, card -> dmas[ ULTRA_GUS_DMA_GRECORD ].dma );
#endif
  if ( gus -> equal_dma || !i )
    {
      if ( !i )
        PRINTK( "ultra: Warning! DMA2 isn't defined.\n" );
      dma |= 0x40;
    }
   else
    dma |= i << 3;
    
  if ( ( dma & 7 ) == 0 )
    {
      PRINTK( "ultra: Error! DMA isn't defined.\n" );
      return -EINVAL;
    }

  irq = irqs[ gus -> gf1.irq & 0x0f ];
  if ( !irq )
    {
      PRINTK( "ultra: Error! IRQ isn't defined.\n" );
      return -EINVAL;
    }
  irq |= 0x40;
#if 0
  card -> mixer.mix_ctrl_reg |= 0x10;
#endif

  CLI( &flags );
  OUTB( 5, GUSP( gus, REGCNTRLS ) );
  OUTB( gus -> mix_cntrl_reg, GUSP( gus, MIXCNTRLREG ) );
  OUTB( 0x00, GUSP( gus, IRQDMACNTRLREG ) );
  OUTB( 0, GUSP( gus, REGCNTRLS ) );
  STI( &flags );

  ultra_gf1_delay( gus );

  CLI( &flags );
  OUTB( 0x00 | gus -> mix_cntrl_reg, GUSP( gus, MIXCNTRLREG ) );
  OUTB( dma, GUSP( gus, IRQDMACNTRLREG ) );
  if ( latches )
    {
      OUTB( 0x40 | gus -> mix_cntrl_reg, GUSP( gus, MIXCNTRLREG ) );
      OUTB( irq, GUSP( gus, IRQDMACNTRLREG ) );
    }
  STI( &flags );

  ultra_gf1_delay( gus );
  
  CLI( &flags );
  OUTB( 0x00 | gus -> mix_cntrl_reg, GUSP( gus, MIXCNTRLREG ) );
  OUTB( dma, GUSP( gus, IRQDMACNTRLREG ) );
  if ( latches )
    {
      OUTB( 0x40 | gus -> mix_cntrl_reg, GUSP( gus, MIXCNTRLREG ) );
      OUTB( irq, GUSP( gus, IRQDMACNTRLREG ) );
    }
  STI( &flags );

  ultra_gf1_delay( gus );
  
  if ( latches )
    gus -> mix_cntrl_reg |= 0x08;	/* enable latches */
   else
    gus -> mix_cntrl_reg &= ~0x08;	/* disable latches */
  CLI( &flags );
  OUTB( gus -> mix_cntrl_reg, GUSP( gus, MIXCNTRLREG ) );
  OUTB( 0, GUSP( gus, GF1PAGE ) );
  STI( &flags );
  
  return 0; 
}
